@@ -1,8 +1,8 @@
 /*
 **  OSSP fsl - Faking/Flexible Syslog Library
-**  Copyright (c) 2002 Ralf S. Engelschall <rse@engelschall.com>
-**  Copyright (c) 2002 The OSSP Project <http://www.ossp.org/>
-**  Copyright (c) 2002 Cable & Wireless Deutschland <http://www.cw.com/de/>
+**  Copyright (c) 2002-2003 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 2002-2003 The OSSP Project <http://www.ossp.org/>
+**  Copyright (c) 2002-2003 Cable & Wireless Deutschland <http://www.cw.com/de/>
 **
 **  This file is part of OSSP fsl, a syslog(3) API faking library which
 **  can be found at http://www.ossp.org/pkg/lib/fsl/.
@@ -232,7 +232,7 @@ static fsl_rc_t appendfiletobuffer(buf_t *buffer, const char *filename)
     if (filename == NULL || buffer == NULL)
         CU(FSL_ERR_ARG);
 
-    fsldebug(L2_LEVEL_TRACE, "appendfiletobuffer() filename=\"%s\")", filename);
+    fsldebug(L2_LEVEL_TRACE, "appendfiletobuffer() filename=\"%s\"", filename);
 
     if ((fd = open(filename, O_RDONLY)) == -1)
         CU(FSL_ERR_SYS);
@@ -496,7 +496,7 @@ static fsl_rc_t processcfg(cfg_t *cfg, char *cpISF, int mode)
         if (cfgnume == 0) {
             fsldebug(L2_LEVEL_ERROR, "processcfg: syntax error, invalid argument \"%s\"", cfgargtoka[0]); CU(FSL_ERR_CUS); }
         if (cfgnume != cfgnumc) {
-            fsldebug(L2_LEVEL_ERROR, "processcfg: directive missing arguments, expected %d, got %d", cfgnume, cfgnumc); CU(FSL_ERR_CUS); }
+            fsldebug(L2_LEVEL_ERROR, "processcfg: invalid number of arguments for directive, expected %d, got %d", cfgnume, cfgnumc); CU(FSL_ERR_CUS); }
 
         for (i = 1; i < cfgnume; i++) {
             /*  process right brother of argument, check if it is an argument and has a valid token attached */
