@@ -1,4 +1,4 @@
-/* A Bison parser, made from l2_spec_parse.y, by GNU bison 1.75.  */
+/* A Bison parser, made by GNU Bison 1.875.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
    Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
@@ -34,10 +34,13 @@
    USER NAME SPACE" below.  */
 
 /* Identify Bison output.  */
-#define YYBISON	1
+#define YYBISON 1
+
+/* Skeleton name.  */
+#define YYSKELETON_NAME "yacc.c"
 
 /* Pure parsers.  */
-#define YYPURE	1
+#define YYPURE 1
 
 /* Using locations.  */
 #define YYLSP_NEEDED 1
@@ -77,11 +80,12 @@
 #line 1 "l2_spec_parse.y"
 
 /*
-**  OSSP l2 - Logging Library
-**  Copyright (c) 2001-2002 The OSSP Project (http://www.ossp.org/)
-**  Copyright (c) 2001-2002 Cable & Wireless Deutschland (http://www.cw.com/de/)
+**  OSSP l2 - Flexible Logging
+**  Copyright (c) 2001-2003 Cable & Wireless Deutschland GmbH
+**  Copyright (c) 2001-2003 The OSSP Project (http://www.ossp.org/)
+**  Copyright (c) 2001-2003 Ralf S. Engelschall <rse@engelschall.com>
 **
-**  This file is part of OSSP L2, a flexible logging library which
+**  This file is part of OSSP l2, a flexible logging library which
 **  can be found at http://www.ossp.org/pkg/lib/l2/.
 **
 **  Permission to use, copy, modify, and distribute this software for
@@ -104,7 +108,7 @@
 **
 **  l2_spec_parse.y: GNU Bison (Yacc-style) parser specification
 **
-**  ATTENTION: This requires GNU Bison 1.30 or newer!
+**  ATTENTION: This requires GNU Bison 1.875 or newer!
 */
 
 #include "l2.h"      /* for l2_xxx() API */
@@ -116,17 +120,16 @@
 #define YYPARSE_PARAM ctx
 #define YYLEX_PARAM   CTX->yyscan
 
-/* provide an explicit prototype for the yylex() function but use 
-   "void *" instead of correct type because at this point in the 
+/* provide an explicit prototype for the yylex() function but use
+   "void *" instead of correct type because at this point in the
    generation process we have the types still not available */
-extern int yylex(/*YYSTYPE*/ void *lvalp, 
+extern int yylex(/*YYSTYPE*/ void *lvalp,
                  /*YYLTYPE*/ void *llocp, l2_spec_ctx_t *ctx);
 
 /* generate verbose error messages and remember them inside the context */
 #undef  yyerror
 #define yyerror(msg) \
     l2_spec_error(CTX, L2_ERR_SYN, &yylloc, msg)
-#define YYERROR_VERBOSE
 
 /* scanner state transition */
 extern void l2_spec_scan_push(l2_spec_ctx_t *, const char *state);
@@ -143,39 +146,42 @@ extern void l2_spec_scan_pop(l2_spec_ctx_t *);
 # undef YYERROR_VERBOSE
 # define YYERROR_VERBOSE 1
 #else
-# define YYERROR_VERBOSE 0
+# define YYERROR_VERBOSE 1
 #endif
 
-#ifndef YYSTYPE
-#line 65 "l2_spec_parse.y"
-typedef union {
+#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
+#line 66 "l2_spec_parse.y"
+typedef union YYSTYPE {
     char         *cpValue;
     l2_channel_t *chChannel;
     unsigned int  nLevels;
-} yystype;
-/* Line 193 of /usr/opkg/share/bison/yacc.c.  */
-#line 158 "l2_spec_parse.c"
-# define YYSTYPE yystype
+} YYSTYPE;
+/* Line 191 of yacc.c.  */
+#line 160 "l2_spec_parse.c"
+# define yystype YYSTYPE /* obsolescent; will be withdrawn */
+# define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
 #endif
 
-#ifndef YYLTYPE
-typedef struct yyltype
+#if ! defined (YYLTYPE) && ! defined (YYLTYPE_IS_DECLARED)
+typedef struct YYLTYPE
 {
   int first_line;
   int first_column;
   int last_line;
   int last_column;
-} yyltype;
-# define YYLTYPE yyltype
+} YYLTYPE;
+# define yyltype YYLTYPE /* obsolescent; will be withdrawn */
+# define YYLTYPE_IS_DECLARED 1
 # define YYLTYPE_IS_TRIVIAL 1
 #endif
 
+
 /* Copy the second part of user declarations.  */
 
 
-/* Line 213 of /usr/opkg/share/bison/yacc.c.  */
-#line 179 "l2_spec_parse.c"
+/* Line 214 of yacc.c.  */
+#line 184 "l2_spec_parse.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -222,13 +228,13 @@ union yyalloc
 };
 
 /* The size of the maximum gap between one aligned stack and the next.  */
-# define YYSTACK_GAP_MAX (sizeof (union yyalloc) - 1)
+# define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
 
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
      ((N) * (sizeof (short) + sizeof (YYSTYPE) + sizeof (YYLTYPE))	\
-      + 2 * YYSTACK_GAP_MAX)
+      + 2 * YYSTACK_GAP_MAXIMUM)
 
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
@@ -242,7 +248,7 @@ union yyalloc
 	{					\
 	  register YYSIZE_T yyi;		\
 	  for (yyi = 0; yyi < (Count); yyi++)	\
-	    (To)[yyi] = (From)[yyi];	\
+	    (To)[yyi] = (From)[yyi];		\
 	}					\
       while (0)
 #  endif
@@ -259,7 +265,7 @@ union yyalloc
 	YYSIZE_T yynewbytes;						\
 	YYCOPY (&yyptr->Stack, Stack, yysize);				\
 	Stack = &yyptr->Stack;						\
-	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAX;	\
+	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
     while (0)
@@ -274,6 +280,7 @@ union yyalloc
 
 /* YYFINAL -- State number of the termination state. */
 #define YYFINAL  11
+/* YYLAST -- Last index in YYTABLE.  */
 #define YYLAST   41
 
 /* YYNTOKENS -- Number of terminals. */
@@ -289,8 +296,8 @@ union yyalloc
 #define YYUNDEFTOK  2
 #define YYMAXUTOK   261
 
-#define YYTRANSLATE(X) \
-  ((unsigned)(X) <= YYMAXUTOK ? yytranslate[X] : YYUNDEFTOK)
+#define YYTRANSLATE(YYX) 						\
+  ((unsigned int) (YYX) <= YYMAXUTOK ? yytranslate[YYX] : YYUNDEFTOK)
 
 /* YYTRANSLATE[YYLEX] -- Bison symbol number corresponding to YYLEX.  */
 static const unsigned char yytranslate[] =
@@ -350,9 +357,9 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned char yyrline[] =
 {
-       0,    96,    96,   103,   111,   114,   121,   132,   135,   146,
-     153,   160,   167,   175,   182,   190,   202,   202,   223,   224,
-     225,   230,   231,   236,   236,   236
+       0,    97,    97,   104,   112,   115,   122,   133,   136,   147,
+     154,   161,   168,   176,   183,   191,   203,   203,   223,   225,
+     226,   231,   232,   237,   237,   237
 };
 #endif
 
@@ -436,7 +443,7 @@ static const yysigned_char yypgoto[] =
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
    positive, shift that token.  If negative, reduce the rule which
    number is the opposite.  If zero, do what YYDEFACT says.
-   If YYTABLE_NINF, parse error.  */
+   If YYTABLE_NINF, syntax error.  */
 #define YYTABLE_NINF -13
 static const yysigned_char yytable[] =
 {
@@ -485,7 +492,7 @@ static const unsigned char yystos[] =
 
 #define yyerrok		(yyerrstatus = 0)
 #define yyclearin	(yychar = YYEMPTY)
-#define YYEMPTY		-2
+#define YYEMPTY		(-2)
 #define YYEOF		0
 
 #define YYACCEPT	goto yyacceptlab
@@ -506,13 +513,13 @@ do								\
     {								\
       yychar = (Token);						\
       yylval = (Value);						\
-      yychar1 = YYTRANSLATE (yychar);				\
+      yytoken = YYTRANSLATE (yychar);				\
       YYPOPSTACK;						\
       goto yybackup;						\
     }								\
   else								\
     { 								\
-      yyerror ("syntax error: cannot back up");			\
+      yyerror ("syntax error: cannot back up");\
       YYERROR;							\
     }								\
 while (0)
@@ -524,7 +531,7 @@ while (0)
    are run).  */
 
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)           \
+# define YYLLOC_DEFAULT(Current, Rhs, N)         \
   Current.first_line   = Rhs[1].first_line;      \
   Current.first_column = Rhs[1].first_column;    \
   Current.last_line    = Rhs[N].last_line;       \
@@ -534,9 +541,9 @@ while (0)
 /* YYLEX -- calling `yylex' with the right arguments.  */
 
 #ifdef YYLEX_PARAM
-# define YYLEX	yylex (&yylval, &yylloc, YYLEX_PARAM)
+# define YYLEX yylex (&yylval, &yylloc, YYLEX_PARAM)
 #else
-# define YYLEX	yylex (&yylval, &yylloc)
+# define YYLEX yylex (&yylval, &yylloc)
 #endif
 
 /* Enable debugging if requested.  */
@@ -552,19 +559,93 @@ do {						\
   if (yydebug)					\
     YYFPRINTF Args;				\
 } while (0)
+
 # define YYDSYMPRINT(Args)			\
 do {						\
   if (yydebug)					\
     yysymprint Args;				\
 } while (0)
+
+# define YYDSYMPRINTF(Title, Token, Value, Location)		\
+do {								\
+  if (yydebug)							\
+    {								\
+      YYFPRINTF (stderr, "%s ", Title);				\
+      yysymprint (stderr, 					\
+                  Token, Value, Location);	\
+      YYFPRINTF (stderr, "\n");					\
+    }								\
+} while (0)
+
+/*------------------------------------------------------------------.
+| yy_stack_print -- Print the state stack from its BOTTOM up to its |
+| TOP (cinluded).                                                   |
+`------------------------------------------------------------------*/
+
+#if defined (__STDC__) || defined (__cplusplus)
+static void
+yy_stack_print (short *bottom, short *top)
+#else
+static void
+yy_stack_print (bottom, top)
+    short *bottom;
+    short *top;
+#endif
+{
+  YYFPRINTF (stderr, "Stack now");
+  for (/* Nothing. */; bottom <= top; ++bottom)
+    YYFPRINTF (stderr, " %d", *bottom);
+  YYFPRINTF (stderr, "\n");
+}
+
+# define YY_STACK_PRINT(Bottom, Top)				\
+do {								\
+  if (yydebug)							\
+    yy_stack_print ((Bottom), (Top));				\
+} while (0)
+
+
+/*------------------------------------------------.
+| Report that the YYRULE is going to be reduced.  |
+`------------------------------------------------*/
+
+#if defined (__STDC__) || defined (__cplusplus)
+static void
+yy_reduce_print (int yyrule)
+#else
+static void
+yy_reduce_print (yyrule)
+    int yyrule;
+#endif
+{
+  int yyi;
+  unsigned int yylineno = yyrline[yyrule];
+  YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
+             yyrule - 1, yylineno);
+  /* Print the symbols being reduced, and their result.  */
+  for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
+    YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
+  YYFPRINTF (stderr, "-> %s\n", yytname [yyr1[yyrule]]);
+}
+
+# define YY_REDUCE_PRINT(Rule)		\
+do {					\
+  if (yydebug)				\
+    yy_reduce_print (Rule);		\
+} while (0)
+
 /* Nonzero means print parse trace.  It is left uninitialized so that
    multiple parsers can coexist.  */
 int yydebug;
 #else /* !YYDEBUG */
 # define YYDPRINTF(Args)
 # define YYDSYMPRINT(Args)
+# define YYDSYMPRINTF(Title, Token, Value, Location)
+# define YY_STACK_PRINT(Bottom, Top)
+# define YY_REDUCE_PRINT(Rule)
 #endif /* !YYDEBUG */
 
+
 /* YYINITDEPTH -- initial size of the parser's stacks.  */
 #ifndef	YYINITDEPTH
 # define YYINITDEPTH 200
@@ -643,106 +724,115 @@ yystpcpy (yydest, yysrc)
 
 
 #if YYDEBUG
-/*-----------------------------.
-| Print this symbol on YYOUT.  |
-`-----------------------------*/
+/*--------------------------------.
+| Print this symbol on YYOUTPUT.  |
+`--------------------------------*/
 
-static void
 #if defined (__STDC__) || defined (__cplusplus)
-yysymprint (FILE* yyout, int yytype, YYSTYPE yyvalue, YYLTYPE yylocation)
+static void
+yysymprint (FILE *yyoutput, int yytype, YYSTYPE *yyvaluep, YYLTYPE *yylocationp)
 #else
-yysymprint (yyout, yytype, yyvalue, yylocation)
-    FILE* yyout;
+static void
+yysymprint (yyoutput, yytype, yyvaluep, yylocationp)
+    FILE *yyoutput;
     int yytype;
-    YYSTYPE yyvalue;
-    YYLTYPE yylocation;
+    YYSTYPE *yyvaluep;
+    YYLTYPE *yylocationp;
 #endif
 {
   /* Pacify ``unused variable'' warnings.  */
-  (void) yyvalue;
-  (void) yylocation;
+  (void) yyvaluep;
+  (void) yylocationp;
 
   if (yytype < YYNTOKENS)
     {
-      YYFPRINTF (yyout, "token %s (", yytname[yytype]);
+      YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
 # ifdef YYPRINT
-      YYPRINT (yyout, yytoknum[yytype], yyvalue);
+      YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
 # endif
     }
   else
-    YYFPRINTF (yyout, "nterm %s (", yytname[yytype]);
+    YYFPRINTF (yyoutput, "nterm %s (", yytname[yytype]);
 
   switch (yytype)
     {
       default:
         break;
     }
-  YYFPRINTF (yyout, ")");
+  YYFPRINTF (yyoutput, ")");
 }
-#endif /* YYDEBUG. */
-
 
+#endif /* ! YYDEBUG */
 /*-----------------------------------------------.
 | Release the memory associated to this symbol.  |
 `-----------------------------------------------*/
 
-static void
 #if defined (__STDC__) || defined (__cplusplus)
-yydestruct (int yytype, YYSTYPE yyvalue, YYLTYPE yylocation)
+static void
+yydestruct (int yytype, YYSTYPE *yyvaluep, YYLTYPE *yylocationp)
 #else
-yydestruct (yytype, yyvalue, yylocation)
+static void
+yydestruct (yytype, yyvaluep, yylocationp)
     int yytype;
-    YYSTYPE yyvalue;
-    YYLTYPE yylocation;
+    YYSTYPE *yyvaluep;
+    YYLTYPE *yylocationp;
 #endif
 {
   /* Pacify ``unused variable'' warnings.  */
-  (void) yyvalue;
-  (void) yylocation;
+  (void) yyvaluep;
+  (void) yylocationp;
 
   switch (yytype)
     {
+
       default:
         break;
     }
 }
-
 
 
-/* The user can define YYPARSE_PARAM as the name of an argument to be passed
-   into yyparse.  The argument should have type void *.
-   It should actually point to an object.
-   Grammar actions can access the variable by casting it
-   to the proper pointer type.  */
+/* Prevent warnings from -Wmissing-prototypes.  */
 
 #ifdef YYPARSE_PARAM
 # if defined (__STDC__) || defined (__cplusplus)
-#  define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
-#  define YYPARSE_PARAM_DECL
+int yyparse (void *YYPARSE_PARAM);
 # else
-#  define YYPARSE_PARAM_ARG YYPARSE_PARAM
-#  define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
+int yyparse ();
 # endif
-#else /* !YYPARSE_PARAM */
-# define YYPARSE_PARAM_ARG
-# define YYPARSE_PARAM_DECL
-#endif /* !YYPARSE_PARAM */
-
-/* Prevent warning if -Wstrict-prototypes.  */
-#ifdef __GNUC__
-# ifdef YYPARSE_PARAM
-int yyparse (void *);
-# else
+#else /* ! YYPARSE_PARAM */
+#if defined (__STDC__) || defined (__cplusplus)
 int yyparse (void);
-# endif
+#else
+int yyparse ();
 #endif
+#endif /* ! YYPARSE_PARAM */
 
 
 
 
+
+
+/*----------.
+| yyparse.  |
+`----------*/
+
+#ifdef YYPARSE_PARAM
+# if defined (__STDC__) || defined (__cplusplus)
+int yyparse (void *YYPARSE_PARAM)
+# else
+int yyparse (YYPARSE_PARAM)
+  void *YYPARSE_PARAM;
+# endif
+#else /* ! YYPARSE_PARAM */
+#if defined (__STDC__) || defined (__cplusplus)
 int
-yyparse (YYPARSE_PARAM_ARG)
-     YYPARSE_PARAM_DECL
+yyparse (void)
+#else
+int
+yyparse ()
+
+#endif
+#endif
 {
   /* The lookahead symbol.  */
 int yychar;
@@ -750,7 +840,7 @@ int yychar;
 /* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
-/* Number of parse errors so far.  */
+/* Number of syntax errors so far.  */
 int yynerrs;
 /* Location data for the lookahead symbol.  */
 YYLTYPE yylloc;
@@ -761,7 +851,7 @@ YYLTYPE yylloc;
   /* Number of tokens to shift before error messages enabled.  */
   int yyerrstatus;
   /* Lookahead token as an internal (translated) token number.  */
-  int yychar1 = 0;
+  int yytoken = 0;
 
   /* Three stacks and their tools:
      `yyss': related to states,
@@ -785,6 +875,7 @@ YYLTYPE yylloc;
   YYLTYPE yylsa[YYINITDEPTH];
   YYLTYPE *yyls = yylsa;
   YYLTYPE *yylsp;
+  YYLTYPE *yylerrsp;
 
 #define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
 
@@ -828,7 +919,7 @@ YYLTYPE yylloc;
  yysetstate:
   *yyssp = yystate;
 
-  if (yyssp >= yyss + yystacksize - 1)
+  if (yyss + yystacksize - 1 <= yyssp)
     {
       /* Get the current used size of the three stacks, in elements.  */
       YYSIZE_T yysize = yyssp - yyss + 1;
@@ -860,10 +951,10 @@ YYLTYPE yylloc;
       goto yyoverflowlab;
 # else
       /* Extend the stack our own way.  */
-      if (yystacksize >= YYMAXDEPTH)
+      if (YYMAXDEPTH <= yystacksize)
 	goto yyoverflowlab;
       yystacksize *= 2;
-      if (yystacksize > YYMAXDEPTH)
+      if (YYMAXDEPTH < yystacksize)
 	yystacksize = YYMAXDEPTH;
 
       {
@@ -889,7 +980,7 @@ YYLTYPE yylloc;
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
-      if (yyssp >= yyss + yystacksize - 1)
+      if (yyss + yystacksize - 1 <= yyssp)
 	YYABORT;
     }
 
@@ -914,39 +1005,28 @@ yybackup:
 
   /* Not known => get a lookahead token if don't already have one.  */
 
-  /* yychar is either YYEMPTY or YYEOF
-     or a valid token in external form.  */
-
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
       yychar = YYLEX;
     }
 
-  /* Convert token to internal form (in yychar1) for indexing tables with.  */
-
-  if (yychar <= 0)		/* This means end of input.  */
+  if (yychar <= YYEOF)
     {
-      yychar1 = 0;
-      yychar = YYEOF;		/* Don't call YYLEX any more.  */
-
+      yychar = yytoken = YYEOF;
       YYDPRINTF ((stderr, "Now at end of input.\n"));
     }
   else
     {
-      yychar1 = YYTRANSLATE (yychar);
-
-      /* We have to keep this `#if YYDEBUG', since we use variables
-	 which are defined only if `YYDEBUG' is set.  */
-      YYDPRINTF ((stderr, "Next token is "));
-      YYDSYMPRINT ((stderr, yychar1, yylval, yyloc));
-      YYDPRINTF ((stderr, "\n"));
+      yytoken = YYTRANSLATE (yychar);
+      YYDSYMPRINTF ("Next token is", yytoken, &yylval, &yylloc);
     }
 
-  /* If the proper action on seeing token YYCHAR1 is to reduce or to
+  /* If the proper action on seeing token YYTOKEN is to reduce or to
      detect an error, take that action.  */
-  yyn += yychar1;
-  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yychar1)
+  yyn += yytoken;
+  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
     goto yydefault;
   yyn = yytable[yyn];
   if (yyn <= 0)
@@ -961,8 +1041,7 @@ yybackup:
     YYACCEPT;
 
   /* Shift the lookahead token.  */
-  YYDPRINTF ((stderr, "Shifting token %d (%s), ",
-	      yychar, yytname[yychar1]));
+  YYDPRINTF ((stderr, "Shifting token %s, ", yytname[yytoken]));
 
   /* Discard the token being shifted unless it is eof.  */
   if (yychar != YYEOF)
@@ -1009,34 +1088,18 @@ yyreduce:
 
   /* Default location. */
   YYLLOC_DEFAULT (yyloc, (yylsp - yylen), yylen);
-
-#if YYDEBUG
-  /* We have to keep this `#if YYDEBUG', since we use variables which
-     are defined only if `YYDEBUG' is set.  */
-  if (yydebug)
-    {
-      int yyi;
-
-      YYFPRINTF (stderr, "Reducing via rule %d (line %d), ",
-		 yyn - 1, yyrline[yyn]);
-
-      /* Print the symbols being reduced, and their result.  */
-      for (yyi = yyprhs[yyn]; yyrhs[yyi] >= 0; yyi++)
-	YYFPRINTF (stderr, "%s ", yytname[yyrhs[yyi]]);
-      YYFPRINTF (stderr, " -> %s\n", yytname[yyr1[yyn]]);
-    }
-#endif
+  YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
         case 2:
-#line 96 "l2_spec_parse.y"
-    { 
+#line 97 "l2_spec_parse.y"
+    {
           CTX->ch = yyvsp[0].chChannel;
-      }
+      ;}
     break;
 
   case 3:
-#line 103 "l2_spec_parse.y"
+#line 104 "l2_spec_parse.y"
     {
           l2_channel_t *ch;
           if ((CTX->rv = l2_channel_create(&ch, CTX->env, "null")) != L2_OK) {
@@ -1044,131 +1107,131 @@ yyreduce:
               YYERROR;
           }
           yyval.chChannel = ch;
-      }
+      ;}
     break;
 
   case 4:
-#line 111 "l2_spec_parse.y"
-    { 
-          yyval.chChannel = yyvsp[0].chChannel; 
-      }
+#line 112 "l2_spec_parse.y"
+    {
+          yyval.chChannel = yyvsp[0].chChannel;
+      ;}
     break;
 
   case 5:
-#line 114 "l2_spec_parse.y"
-    { 
-          yyval.chChannel = yyvsp[-2].chChannel; 
+#line 115 "l2_spec_parse.y"
+    {
+          yyval.chChannel = yyvsp[-2].chChannel;
           if ((CTX->rv = l2_channel_link(yyvsp[-2].chChannel, L2_LINK_CHILD, yyvsp[0].chChannel, NULL)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "unable to link parent with child");
               YYERROR;
           }
-      }
+      ;}
     break;
 
   case 6:
-#line 121 "l2_spec_parse.y"
-    { 
-          yyval.chChannel = yyvsp[-4].chChannel; 
+#line 122 "l2_spec_parse.y"
+    {
+          yyval.chChannel = yyvsp[-4].chChannel;
           if ((CTX->rv = l2_channel_link(yyvsp[-4].chChannel, L2_LINK_CHILD, yyvsp[-1].chChannel, NULL)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "unable to link parent with child list");
               YYERROR;
           }
-      }
+      ;}
     break;
 
   case 7:
-#line 132 "l2_spec_parse.y"
-    { 
-          yyval.chChannel = yyvsp[0].chChannel; 
-      }
+#line 133 "l2_spec_parse.y"
+    {
+          yyval.chChannel = yyvsp[0].chChannel;
+      ;}
     break;
 
   case 8:
-#line 135 "l2_spec_parse.y"
-    { 
-          yyval.chChannel = yyvsp[-2].chChannel; 
+#line 136 "l2_spec_parse.y"
+    {
+          yyval.chChannel = yyvsp[-2].chChannel;
           if ((CTX->rv = l2_channel_link(yyvsp[-2].chChannel, L2_LINK_SIBLING, yyvsp[0].chChannel, NULL)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "unable to link childs together");
               YYERROR;
           }
-      }
+      ;}
     break;
 
   case 9:
-#line 146 "l2_spec_parse.y"
-    { 
-          yyval.chChannel = yyvsp[0].chChannel; 
+#line 147 "l2_spec_parse.y"
+    {
+          yyval.chChannel = yyvsp[0].chChannel;
           if ((CTX->rv = l2_channel_levels(yyvsp[0].chChannel, yyvsp[-4].nLevels, yyvsp[-2].nLevels)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "failed to set channel write and flush levels");
               YYERROR;
           }
-      }
+      ;}
     break;
 
   case 10:
-#line 153 "l2_spec_parse.y"
-    { 
-          yyval.chChannel = yyvsp[0].chChannel; 
+#line 154 "l2_spec_parse.y"
+    {
+          yyval.chChannel = yyvsp[0].chChannel;
           if ((CTX->rv = l2_channel_levels(yyvsp[0].chChannel, yyvsp[-2].nLevels, L2_LEVEL_NONE)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "failed to set channel write levels");
               YYERROR;
           }
-      }
+      ;}
     break;
 
   case 11:
-#line 160 "l2_spec_parse.y"
-    { 
+#line 161 "l2_spec_parse.y"
+    {
           yyval.chChannel = yyvsp[0].chChannel;
-      }
+      ;}
     break;
 
   case 12:
-#line 167 "l2_spec_parse.y"
-    { 
+#line 168 "l2_spec_parse.y"
+    {
           unsigned int levelmask;
           if ((CTX->rv = l2_util_s2l(yyvsp[0].cpValue, strlen(yyvsp[0].cpValue), ',', &levelmask)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "invalid level '%s'", yyvsp[0].cpValue);
               YYERROR;
           }
           yyval.nLevels = L2_LEVEL_UPTO(levelmask);
-      }
+      ;}
     break;
 
   case 13:
-#line 175 "l2_spec_parse.y"
-    { 
-          yyval.nLevels = yyvsp[-1].nLevels; 
-      }
+#line 176 "l2_spec_parse.y"
+    {
+          yyval.nLevels = yyvsp[-1].nLevels;
+      ;}
     break;
 
   case 14:
-#line 182 "l2_spec_parse.y"
-    { 
+#line 183 "l2_spec_parse.y"
+    {
           unsigned int levelmask;
           if ((CTX->rv = l2_util_s2l(yyvsp[0].cpValue, strlen(yyvsp[0].cpValue), ',', &levelmask)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "invalid level '%s'", yyvsp[0].cpValue);
               YYERROR;
           }
           yyval.nLevels = levelmask;
-      }
+      ;}
     break;
 
   case 15:
-#line 190 "l2_spec_parse.y"
-    { 
+#line 191 "l2_spec_parse.y"
+    {
           unsigned int levelmask;
           if ((CTX->rv = l2_util_s2l(yyvsp[-2].cpValue, strlen(yyvsp[-2].cpValue), ',', &levelmask)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "invalid level '%s'", yyvsp[-2].cpValue);
               YYERROR;
           }
-          yyval.nLevels = levelmask | yyvsp[0].nLevels; 
-      }
+          yyval.nLevels = levelmask | yyvsp[0].nLevels;
+      ;}
     break;
 
   case 16:
-#line 202 "l2_spec_parse.y"
-    { 
+#line 203 "l2_spec_parse.y"
+    {
           l2_channel_t *ch;
           if ((CTX->rv = l2_channel_create(&ch, CTX->env, yyvsp[0].cpValue)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "failed to create channel '%s'", yyvsp[0].cpValue);
@@ -1179,58 +1242,49 @@ yyreduce:
              not know where on the token stack our $$ is because it is a
              sub-rule of the recursive channel_param_list rule and hence
              cannot use "$<chChannel>-n". */
-          CTX->chTmp = ch; 
-      }
+          CTX->chTmp = ch;
+      ;}
     break;
 
   case 17:
-#line 215 "l2_spec_parse.y"
-    { 
+#line 216 "l2_spec_parse.y"
+    {
           yyval.chChannel = yyvsp[-1].chChannel; /* pass-through result */
           CTX->chTmp = NULL;
-      }
+      ;}
     break;
 
   case 23:
-#line 236 "l2_spec_parse.y"
-    { l2_spec_scan_push(CTX, "SS_PARAM"); }
+#line 237 "l2_spec_parse.y"
+    { l2_spec_scan_push(CTX, "SS_PARAM"); ;}
     break;
 
   case 24:
-#line 236 "l2_spec_parse.y"
-    { l2_spec_scan_pop(CTX); }
+#line 237 "l2_spec_parse.y"
+    { l2_spec_scan_pop(CTX); ;}
     break;
 
   case 25:
-#line 236 "l2_spec_parse.y"
-    { 
+#line 237 "l2_spec_parse.y"
+    {
           if ((CTX->rv = l2_channel_configure(CTX->chTmp, "%s=\"%s\"", yyvsp[-4].cpValue, yyvsp[-1].cpValue)) != L2_OK) {
               l2_spec_error(CTX, CTX->rv, &yylloc, "failed to configure channel with '%s=\"%s\"'", yyvsp[-4].cpValue, yyvsp[-1].cpValue);
               YYERROR;
           }
-      }
+      ;}
     break;
 
 
     }
 
-/* Line 1016 of /usr/opkg/share/bison/yacc.c.  */
-#line 1219 "l2_spec_parse.c"
+/* Line 991 of yacc.c.  */
+#line 1281 "l2_spec_parse.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
   yylsp -= yylen;
 
-#if YYDEBUG
-  if (yydebug)
-    {
-      short *yyssp1 = yyss - 1;
-      YYFPRINTF (stderr, "state stack now");
-      while (yyssp1 != yyssp)
-	YYFPRINTF (stderr, " %d", *++yyssp1);
-      YYFPRINTF (stderr, "\n");
-    }
-#endif
+  YY_STACK_PRINT (yyss, yyssp);
 
   *++yyvsp = yyval;
   *++yylsp = yyloc;
@@ -1275,12 +1329,12 @@ yyerrlab:
 	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
-	  yysize += yystrlen ("parse error, unexpected ") + 1;
+	  yysize += yystrlen ("syntax error, unexpected ") + 1;
 	  yysize += yystrlen (yytname[yytype]);
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
-	      char *yyp = yystpcpy (yymsg, "parse error, unexpected ");
+	      char *yyp = yystpcpy (yymsg, "syntax error, unexpected ");
 	      yyp = yystpcpy (yyp, yytname[yytype]);
 
 	      if (yycount < 5)
@@ -1301,19 +1355,15 @@ yyerrlab:
 	      YYSTACK_FREE (yymsg);
 	    }
 	  else
-	    yyerror ("parse error; also virtual memory exhausted");
+	    yyerror ("syntax error; also virtual memory exhausted");
 	}
       else
 #endif /* YYERROR_VERBOSE */
-	yyerror ("parse error");
+	yyerror ("syntax error");
     }
-  goto yyerrlab1;
 
+  yylerrsp = yylsp;
 
-/*----------------------------------------------------.
-| yyerrlab1 -- error raised explicitly by an action.  |
-`----------------------------------------------------*/
-yyerrlab1:
   if (yyerrstatus == 3)
     {
       /* If just tried and failed to reuse lookahead token after an
@@ -1325,28 +1375,46 @@ yyerrlab1:
 	  /* Pop the error token.  */
           YYPOPSTACK;
 	  /* Pop the rest of the stack.  */
-	  while (yyssp > yyss)
+	  while (yyss < yyssp)
 	    {
-	      YYDPRINTF ((stderr, "Error: popping "));
-	      YYDSYMPRINT ((stderr,
-			    yystos[*yyssp],
-			    *yyvsp, *yylsp));
-	      YYDPRINTF ((stderr, "\n"));
-	      yydestruct (yystos[*yyssp], *yyvsp, *yylsp);
+	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+	      yydestruct (yystos[*yyssp], yyvsp, yylsp);
 	      YYPOPSTACK;
 	    }
 	  YYABORT;
         }
 
-      YYDPRINTF ((stderr, "Discarding token %d (%s).\n",
-		  yychar, yytname[yychar1]));
-      yydestruct (yychar1, yylval, yylloc);
+      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+      yydestruct (yytoken, &yylval, &yylloc);
       yychar = YYEMPTY;
+      *++yylerrsp = yylloc;
     }
 
   /* Else will try to reuse lookahead token after shifting the error
      token.  */
+  goto yyerrlab2;
 
+
+/*----------------------------------------------------.
+| yyerrlab1 -- error raised explicitly by an action.  |
+`----------------------------------------------------*/
+yyerrlab1:
+
+  /* Suppress GCC warning that yyerrlab1 is unused when no action
+     invokes YYERROR.  */
+#if defined (__GNUC_MINOR__) && 2093 <= (__GNUC__ * 1000 + __GNUC_MINOR__)
+  __attribute__ ((__unused__))
+#endif
+
+  yylerrsp = yylsp;
+  *++yylerrsp = yyloc;
+  goto yyerrlab2;
+
+
+/*---------------------------------------------------------------.
+| yyerrlab2 -- pop states until the error token can be shifted.  |
+`---------------------------------------------------------------*/
+yyerrlab2:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
   for (;;)
@@ -1367,26 +1435,12 @@ yyerrlab1:
       if (yyssp == yyss)
 	YYABORT;
 
-      YYDPRINTF ((stderr, "Error: popping "));
-      YYDSYMPRINT ((stderr,
-		    yystos[*yyssp], *yyvsp, *yylsp));
-      YYDPRINTF ((stderr, "\n"));
-
-      yydestruct (yystos[yystate], *yyvsp, *yylsp);
+      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+      yydestruct (yystos[yystate], yyvsp, yylsp);
       yyvsp--;
       yystate = *--yyssp;
       yylsp--;
-
-#if YYDEBUG
-      if (yydebug)
-	{
-	  short *yyssp1 = yyss - 1;
-	  YYFPRINTF (stderr, "Error: state stack now");
-	  while (yyssp1 != yyssp)
-	    YYFPRINTF (stderr, " %d", *++yyssp1);
-	  YYFPRINTF (stderr, "\n");
-	}
-#endif
+      YY_STACK_PRINT (yyss, yyssp);
     }
 
   if (yyn == YYFINAL)
@@ -1395,7 +1449,8 @@ yyerrlab1:
   YYDPRINTF ((stderr, "Shifting error token, "));
 
   *++yyvsp = yylval;
-  *++yylsp = yylloc;
+  YYLLOC_DEFAULT (yyloc, yylsp, (yylerrsp - yylsp));
+  *++yylsp = yyloc;
 
   yystate = yyn;
   goto yynewstate;
@@ -1434,7 +1489,7 @@ yyreturn:
 }
 
 
-#line 244 "l2_spec_parse.y"
+#line 245 "l2_spec_parse.y"
 
 
 
